# Book: SENSORY EVALUATION OF SOUND, Editor: Nick  Zacharov

# DATA SET for Chapter 12: Concert hall acoustics
# Authors: Antti Kuusinen and Tapio Lokki

# DESCRIPTION OF THE CONCERT HALL ACOUSTICS PAIRED COMPARISON PREFERENCES DATA SET:

# BRIEF: 
# Preferences of concert hall acoustics were collected by the paired comparison method: a sequence of stimulus pairs was presented to the assessors, who indicated the preference for one or the another in each pair. No ties were allowed, and thus, it is the same as the 2AFC protocol. However, as seen below, this data set has a special structure.

# 6 concert halls were compared in 3 seating positions with 2 different music pieces. Each comparison was made twice. Halls were compared against each other in all positions, but comparison were NOT made between the positions, NOR between the music pieces. Thus, each position-music combination can be regarded as one "context" or "criterium" in which the preferences were collected, and so, there are 6 separate contexts in this data set. Accordingly, there are 6 different .csv -files containing the data. One could analyze the 6 data sets separately, but here we have opted for an latent class approach that estimates the probabilities by which each assessor belongs to a latent class, taking into account this special structure of the data. The appropriate number of latent classes also needs to be determined by using the information criteria for model selection.

# 28 assessors participated in the experiment, so, there are 6 by 6 square matrices for each assessor, where each cell denotes the preference of hall in the row over the hall in the column. In each .csv-file, these data matrices are concatenated in rows, so, the .csv-file is a data matrix with 6 columns (halls, no column names), and 168 rows (6*28). This is the structure that ImportData()-function requires. Note, that ImportData()-function will read all 6 .csv-files together when named appropriately. 

# Please refer to the documentation of the CompR-package (and ggplot2 etc.) for the details of the methods!
# https://cran.r-project.org/web/packages/CompR/index.html

# DATA SET SUMMARY:
# 6 halls coded as 'AC', 'BK', 'BP', 'CP', 'HM', 'VM'
# 3 seating positions: R2, R6, R8 ("R" comes from "receiver”, R2 = Front stalls, R6 = First row of balcony, R8 = Parterre)
# 2 music pieces: 'Bruckner' and 'Beethoven'.
# 28 assessors
# Each comparison was made 2 times, so the counts in each cell range from 0-2.

# files: context (music-position): 'BrucknerR2','BrucknerR6','BrucknerR8','BeethovenR2','BeethovenR6','BeethovenR8',
# naming: prefMCMusicPosition.
# for example, prefMCBrucknerR2.csv, data matrix of 168 rows, and 6 cols.


##################################################################################
# Packages:
library(CompR)
library(ggplot2); library(reshape2); library(lattice)

# Change the working directory to the folder where the data is located!!!!
# setwd('.../Book_Data/')


# The following example script produces the plot shown in the book.
##########################################################
############### Begin example: ###########################
# Read the data:
prefMC <- ImportData(name = 'prefMC', labelprod = F, sep = ",") 

# Run the analysis algorithm:
prefpl <- EstimBradley(prefMC, Tcla = 5) 
# Tcla = 2: Two latent classes seems as appropriate, from analyses of the model fits, 
# but one can determine here the number of classes that are extracted. 
# Try different number of classes from 1-5 to see how the algorithm breaks down the people into groups.

# Get the Bradley scores:
prefpl_SCORES <- sapply(getPi(prefpl), FUN = 'matrix')

# Confidence intervals:
prefpl_SDs <- sqrt(sapply(sapply(getVarcov(prefpl),FUN = 'matrix'), FUN = 'diag'))

# Data frame construction for ggplot:
prefHall <- rep(c('AC', 'BK', 'BP', 'CP', 'HM', 'VM'),6)
prefCrit <- rep(c('BrucknerR2','BrucknerR6','BrucknerR8','BeethovenR2','BeethovenR6','BeethovenR8'), each = 6)
prefSeat <- rep(c('R2', 'R6', 'R8','R2', 'R6', 'R8'), each = 6)
prefMus <- rep(c('Bruckner','Bruckner','Bruckner', 'Beethoven','Beethoven','Beethoven'), each = 6)
prefFRAME <- data.frame(prefCrit, prefHall, prefMus,prefSeat, melt(prefpl_SCORES), as.numeric(melt(prefpl_SDs)[,3]) ,melt(prefpl_SCORES)[3]-1.96*as.numeric(melt(prefpl_SDs)[,3]), melt(prefpl_SCORES)[3]+1.96*as.numeric(melt(prefpl_SDs)[,3]))
names(prefFRAME) <- c('Crit','Hall', 'Music', 'Seat', 'Ind', 'Class', 'Pi', 'SigmaPi', 'CiL', 'CiH')
prefFRAME$Hall <- factor(prefFRAME$Hall, c('VM', 'AC','BK', 'BP', 'CP', 'HM'))
prefFRAME$Seat <- factor(prefFRAME$Seat, c('R2', 'R8', 'R6'))
prefFRAME$Music <- factor(prefFRAME$Music, c('Bruckner', 'Beethoven'))
levels(prefFRAME$Seat) <- c('STALLS FRONT', 'PARTERRE', 'BALCONY')

levels(prefFRAME$Class) <- c(paste('Class ', 1:length(prefpl@Lambda), ': ', as.character(round(prefpl@Lambda*100,0)), ' %',sep = "")) # percentages of people in each class
#levels(prefFRAME$Class) <- c(paste('Class ', 1:length(prefpl@Lambda), ': ', as.character(round(prefpl@Lambda*28,0)), ' persons',sep = "")) # number of persons in each class

# Plotting:
dodge = position_dodge(width=0.5)
prefFRAME$CiL[prefFRAME$CiL < 0] <- 0 # Bound confidence intervals to 0-1
prefFRAME$CiH[prefFRAME$CiH > 1] <- 1 # Bound confidence intervals to 0-1

# Check prefFRAME: 
prefFRAME

# pLot: 
PRplot <- ggplot(data = prefFRAME, aes(x = Hall, y = Pi, shape = Music, linetype =Music, group = Music, colour = Music)) + geom_point(position = dodge) + geom_line(position = dodge)
PRplot <-  PRplot + geom_errorbar(aes(x = Hall, ymin = CiL, ymax = CiH), width = 0.3, size = 0.3, position = dodge) + scale_colour_manual(values = c('red', 'blue'))
PRplot <- PRplot + facet_grid(Class ~ Seat) + theme_bw() #+ theme(legend.position = c(.45, .4)) 

#show plot:
PRplot

############### End of example: ###########################
##########################################################